<?php

/* Reminder: always indent with 4 spaces (no tabs). */
// +---------------------------------------------------------------------------+
// | Geeklog 1.7                                                               |
// +---------------------------------------------------------------------------+
// | oauthhelper.class.php                                                     |
// | version: 1.0.0                                                            |
// |                                                                           |
// | Geeklog Distributed Authentication Module.                                |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2010 by the following authors:                              |
// |                                                                           |
// | Authors: Hiroron          - hiroron AT hiroron DOT com                    |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+

if (strpos(strtolower($_SERVER['PHP_SELF']), 'oauthhelper.class.php') !== false) {
    die('This file can not be used on its own.');
}

// PEAR class to handle HTTP-Request2
require_once 'HTTP/Request2.php';

class OAuthConsumer {
    protected $consumer = NULL;

    public function OAuthConsumer ($service)
    {
        global $_CONF;
        
        require_once $_CONF['path_system'] . 'classes/oauth/' . $service . '.auth.class.php';
        $serviceclass = $service . 'Consumer';
        $this->consumer = new $serviceclass;
    }

    public function find_identity_info($callback_url, $query) {
        return $this->consumer->find_identity_info($callback_url, $query);
    }

    public function sreq_userinfo_response($query) {
        return $this->consumer->sreq_userinfo_response($query);
    }

    public function getErrorMsg() {
        return $this->consumer->getErrorMsg();
    }

    public function doAction($info) {
        $this->consumer->doAction($info);
    }

    public function getCallback_query_string() {
        return $this->consumer->callback_query_string;
    }

    public function getCancel_query_string() {
        return $this->consumer->cancel_query_string;
    }

}

class OAuthConsumerBaseClass {
    protected $request = '';
    protected $consumer = '';
    protected $errormsg = '';
    protected $shortapi = 'http://api.tr.im/api/trim_url.xml';
    public $consumer_key = '';
    public $consumer_secret = '';
    public $url_requestToken = '';
    public $url_authorize = '';
    public $url_accessToken = '';
    public $url_userinfo = '';
    public $method_requestToken = 'GET';
    public $method_accessToken = 'GET';
    public $method_userinfo = 'GET';
    public $cookietimeout = 300;    // google Callbacks 5min
    public $token = '';
    public $token_secret = '';
    public $callback_query_string = 'oauth_verifier';
    public $cancel_query_string = '';
    
    public function __construct () {
        $httpRequest = new HTTP_Request2;
        $httpRequest->setConfig('ssl_verify_peer', false);
        $httpRequest->setHeader('Accept-Encoding', '.*');
        $this->request = new HTTP_OAuth_Consumer_Request;
        $this->request->accept($httpRequest);
    }

    public function find_identity_info($callback_url, $query) {
        $url = '';
        
        try {
            $this->consumer = new HTTP_OAuth_Consumer($this->consumer_key, $this->consumer_secret);
            $this->consumer->accept($this->request);
            
            $this->consumer->getRequestToken($this->url_requestToken, $callback_url, array(), $this->method_requestToken);
            $timeout = time() + $this->cookietimeout;
            SEC_setCookie('request_token', $this->consumer->getToken(), $timeout);
            SEC_setCookie('request_token_secret', $this->consumer->getTokenSecret(), $timeout);
            
            $url = $this->consumer->getAuthorizeUrl($this->url_authorize);
        } catch (HTTP_OAuth_Consumer_Exception_Invalid_Response $e) {
            $this->errormsg = get_class($e) . ': ' . $e->getBody();
        } catch (Exception $e) {
            $this->errormsg = get_class($e) . ': ' . $e->getMessage();
        }
        return $url;
    }

    public function sreq_userinfo_response($query) {
        $userinfo = array();
        
        try {
            $this->token = $_COOKIE['request_token'];
            $this->token_secret = $_COOKIE['request_token_secret'];
            $verifier = $query[$this->callback_query_string];
            SEC_setCookie($_COOKIE['request_token'], '', time() - 10000);
            SEC_setCookie($_COOKIE['request_token_secret'], '', time() - 10000);
            $this->consumer = new HTTP_OAuth_Consumer($this->consumer_key, $this->consumer_secret, $this->token, $this->token_secret);
            $this->consumer->accept($this->request);
            
            $this->consumer->getAccessToken($this->url_accessToken, $verifier, array(), $this->method_accessToken);
            
            $this->token = $this->consumer->getToken();
            $this->token_secret = $this->consumer->getTokenSecret();
            
            $this->consumer->setToken($this->token);
            $this->consumer->setTokenSecret($this->token_secret);
            
            $response = $this->consumer->sendRequest($this->url_userinfo, array(), $this->method_userinfo);
            if ($response->getStatus() !== 200) {
                $this->errormsg = $response->getStatus() . ' : ' . $response->getBody();
            } else {
                $userinfo = simplexml_load_string($response->getBody());
            }
        } catch (HTTP_OAuth_Consumer_Exception_Invalid_Response $e) {
            $this->errormsg = get_class($e) . ': ' . $e->getBody();
        } catch (Exception $e) {
            $this->errormsg = get_class($e) . ': ' . $e->getMessage();
        }
        return $userinfo;
    }

    public function doAction($info) {
        
        global $_TABLES, $status, $uid, $_CONF;

        // Remote auth precludes usersubmission,
        // and integrates user activation, see?;
        $status = USER_ACCOUNT_ACTIVE;

        $users = $this->_getCreateUserInfo($info);
        $userinfo = $this->_getUpdateUserInfo($info);
        
        $result = DB_query("SELECT uid,status FROM {$_TABLES['users']} WHERE remoteusername = '{$users['remoteusername']}' AND remoteservice = '{$users['remoteservice']}'");
        $tmp = DB_error();
        $nrows = DB_numRows($result);
        if (empty($tmp) && $nrows == 1) {
            list($uid, $status) = DB_fetchArray($result);
        } else {
            // First time login with this Twitter, creating account...
            $status = USER_ACCOUNT_ACTIVE;

            // Check to see if their remoteusername is unique locally
            $checkName = DB_getItem($_TABLES['users'], 'username', "username='{$users['remoteusername']}'");
            if (!empty($checkName)) {
                // no, call custom function.
                if (function_exists('CUSTOM_uniqueRemoteUsername')) {
                    $loginname = CUSTOM_uniqueRemoteUsername($loginname, $remoteservice);
                }
                if ($checkName == $loginname) {
                    $loginname = USER_uniqueUsername($loginname);
                }
            }

            USER_createAccount($users['loginname'], $users['email'], $users['passwd2'], $users['fullname'], $users['homepage'], $users['remoteusername'], $users['remoteservice']);
            $uid = DB_getItem($_TABLES['users'], 'uid', "remoteusername = '{$users['remoteusername']}' AND remoteservice = '{$users['remoteservice']}'");

            // Update users
            if (is_array($users)) {
                $this->_DBupdate_users($uid, $users);
            }
            
            // Update userinfo
            if (is_array($userinfo)) {
                $this->_DBupdate_userinfo($uid, $userinfo);
            }

            // Add to remote users:
            $remote_grp = DB_getItem($_TABLES['groups'], 'grp_id', "grp_name = 'Remote Users'");
            DB_query("INSERT INTO {$_TABLES['group_assignments']} (ug_main_grp_id, ug_uid) VALUES ($remote_grp, $uid)");

            // usercreate after trigger
            if (method_exists($this, '_after_trigger')) {
                $this->_after_trigger($uid, $users, $userinfo);
            }
        }
    }

    public function getErrorMsg() {
        return $this->errormsg;
    }

    protected function _DBupdate_users($uid, $users) {
        global $_TABLES, $_CONF;
        // Store full remote account name:
        $sql = "UPDATE {$_TABLES['users']} SET remoteusername = '{$users['remoteusername']}', remoteservice = '{$users['remoteservice']}', status = 3";
        if (!empty($users['remotephoto'])) {
            // save user photo
            $save_img = $_CONF['path_images'] . 'userphotos/' . $users['loginname'];
            $imgsize = $this->_saveUserPhoto($users['remotephoto'], $save_img);
            if (!empty($imgsize)) {
                $ext = $this->_getImageExt($save_img);
                rename($save_img, $save_img . $ext);
                $imgname = $users['loginname'] . $ext;
                $sql .= ", photo = '{$imgname}'";
            }
        }
        $sql .= " WHERE uid = $uid";
        DB_query($sql);
    }

    protected function _DBupdate_userinfo($uid, $userinfo) {
        global $_TABLES;
        if (!empty($userinfo['about']) || !empty($userinfo['location'])) {
            $sql = "UPDATE {$_TABLES['userinfo']} SET";
            $sql .= !empty($userinfo['about']) ? " about = '{$userinfo['about']}'" : "";
            $sql .= (!empty($userinfo['about']) && !empty($userinfo['location'])) ? "," : "";
            $sql .= !empty($userinfo['location']) ? " location = '{$userinfo['location']}'" : "";
            $sql .= " WHERE uid = $uid";
            DB_query($sql);
        }
    }

    protected function _saveUserPhoto($from, $to) {
        $ret = '';
        require_once 'HTTP/Request.php';
        $req = new HTTP_Request($from);
        $req->addHeader('User-Agent', 'Geeklog/' . VERSION);
        $req->addHeader('Referer', COM_getCurrentUrl());
        $res = $req->sendRequest();
        if( !PEAR::isError($res) ){
            $img = $req->getResponseBody();
            $ret = file_put_contents($to, $img);
        }
        return $ret;
    }

    protected function _getImageExt($img, $dot = true) {
        $size = getimagesize($img);
        switch ($size['mime']) {
            case 'image/gif':
                $ext = 'gif';
                break;
            case 'image/jpeg':
                $ext = 'jpg';
                break;
            case 'image/png':
                $ext = 'png';
                break;
            case 'image/bmp':
                $ext = 'bmp';
                break;
        }
        return ($dot ? '.' : '') . $ext;
    }

    protected function _shorten($url) {
        $this->request->setUrl($this->shortapi.'?url='.$url);
        $this->request->setMethod('GET');
        $response = $this->request->send();
        if ($response->getStatus() !== 200) {
            return $url;
        } else {
            $xml = @simplexml_load_string($response->getBody());
            return $xml->url;
        }
    }

}


?>
